# surface_equation.rb (C) 2008 jcrubio - gmail.com
#
# HISTORY
# 2006-10-25 - Initial release
#

require "sketchup.rb"

include Math

module SurfacePlotter

   @model = Sketchup.active_model
   @entities = @model.entities

   # ===========================================================================
   def self.draw_config
      conf = {}
      prompts = ["X-Axis Range", "Y-Axis Range", "Z-Axis Range", "Label?"]
      choices = ["", "", "", "yes|no"]
      defaults = ["-10..10", "-10..10", "-10..10", "yes"]
      ret = UI.inputbox(prompts, defaults, choices, "Setup Grid")
      conf[:xrange] = eval(ret[0])
      conf[:yrange] = eval(ret[1])
      conf[:zrange] = eval(ret[2])
      conf[:label] = (ret[3]  == "no" ? false : true)
      draw_grid(conf)
   end

   def self.draw_default
      conf = {:label => true}
      conf[:perspective] = false
      conf[:xrange] = -10..10
      conf[:yrange] = -10..10
      conf[:zrange] = -10..10
      draw_grid(conf)
   end

   def self.draw_grid(conf)
      Sketchup.active_model.start_operation "Draw a coordinate system"
      #@model.active_view.camera.perspective = conf[:perspective]
      g = @entities.add_group
      xrange = conf[:xrange]
      yrange = conf[:yrange]
      zrange = conf[:zrange]
      xrange.each do |x|
         g.entities.add_cline([x, yrange.min, 0], [x, yrange.max, 0])
         g.entities.add_cline([x, 0, zrange.min], [x, 0, zrange.max])
         g.entities.add_text(x.to_s, [x, 0]) if conf[:label]
      end
      yrange.each do |y|
         g.entities.add_cline([xrange.min, y, 0], [xrange.max, y, 0])
         g.entities.add_cline([0, y, zrange.min], [0, y, zrange.max])
         g.entities.add_text(y.to_s, [0, y, 0]) if conf[:label]
      end
      zrange.each do |z|
         g.entities.add_cline([xrange.min, 0, z], [xrange.max, 0, z])
         g.entities.add_cline([0, yrange.min, z], [0, yrange.max, z])
         g.entities.add_text(z.to_s, [0, 0, z]) if conf[:label]
      end
      Sketchup.send_action(CMD_VIEW_ISO)
      Sketchup.send_action(CMD_ZOOM_EXTENTS)
      Sketchup.active_model.commit_operation
   end

   # ===========================================================================
   def self.graph
#     ret = UI.inputbox( ["z = "], ["( ((x-x.floor)*2 - 1)**2 - 1 )/3 + 1 * sin(x/2 - y/6)"], "Enter valid Ruby syntax" )
#     ret = UI.inputbox( ["z = "], [" x*y + x*x - 0.5*y*y"], "Enter valid Ruby syntax" )
#     ret = UI.inputbox( ["z = "], [" x*y - x*x/3 - y*y/2"], "Enter valid Ruby syntax" )
#     ret = UI.inputbox( ["z = "], [" x.abs*cos(0.25*x)*cos(0.25*y) "], "Enter valid Ruby syntax" )
#     ret = UI.inputbox( ["z = "], [" (x+1)/(y-2) "], "Enter valid Ruby syntax" )
      ret = UI.inputbox( ["z = "], [" (x+1)/(y-2) "], "Enter valid Ruby syntax" )
      model = Sketchup.active_model
      entities = model.active_entities

      Sketchup.active_model.start_operation "Generate surface"

      g = @entities.add_group
      g.name = "surface"

      # First itertion builds all points for lower y
      p0 = self.compute_points(ret[0],-10..10,-10)

      n = p0.length

      # Now create all faces
      -9.step(10, 0.5) do |y|

        # Build a new line of points
        p1 = p0
        p0 = self.compute_points(ret[0],-10..10,y)

        # Build faces between lines, using uniform x-y tessellation
        0.upto(n-2) do |i|
          begin
            g.entities.add_face(p1[i], p1[i+1], p0[i+1], p1[i] )
            g.entities.add_face(p0[i], p0[i+1], p1[i], p0[i] )
          rescue
            ;
          end
        end

      end
      Sketchup.active_model.commit_operation

   end

   def self.compute_points(eq,xrange,y)
      include Math
      p = Array.new
      xrange.each do |x|
        begin
          z = eval(eq)
          pt = nil
          pt = [x,y,z] if (-100..100) === z
        rescue => exception
          puts "Found an exception(#{exception}), for x(#{x}) and y(#{y})"
          pt = nil
        end
        p << pt
      end
      return p
   end
   # ===========================================================================
   filename = File::basename(__FILE__)
   unless file_loaded?(filename)
      add_separator_to_menu("Plugins")
      graph = UI.menu("Plugins").add_submenu("Surface plotter")
      graph.add_item("Draw standard grid") {SurfacePlotter.draw_default}
      graph.add_item("Draw custom grid")   {SurfacePlotter.draw_config}
      graph.add_item("Plot surface")       {SurfacePlotter.graph}
      file_loaded(filename)
   end

end

# =============================================================================#
# vim:ts=2:sw=2:expandtab:cindent
